    // BinaryHeap class
    //
    // CONSTRUCTION: with optional capacity (that defaults to 100)
    //
    // ******************PUBLIC OPERATIONS*********************
    // void insert( x )       --> Insert x
    // Comparable deleteMin( )--> Return and remove smallest item
    // Comparable findMin( )  --> Return smallest item
    // boolean isEmpty( )     --> Return true if empty; else false
    // boolean isFull( )      --> Return true if full; else false
    // void makeEmpty( )      --> Remove all items
    // ******************ERRORS********************************

    /**
     * Implements a binary heap.
     * Note that all "matching" is based on the compareTo method.
     * @author Mark Allen Weiss
     */
    public class BinaryHeap
    {
		public interface HeapElement{
			public int comparePriority(HeapElement x);
			public boolean decreasePriority(int value);
			public void setHeapPos(int value);
			public int getHeapPos();
			public int getPriority();
			
	}
        /**
         * Construct the binary heap.
         */
        public BinaryHeap( )
        {
            this( DEFAULT_CAPACITY );
        }

        /**
         * Construct the binary heap.
         * @param capacity the capacity of the binary heap.
         */
        public BinaryHeap( int capacity )
        {
            currentSize = 0;
            array = new HeapElement[ capacity + 1 ];
        }

        /**
         * Insert into the priority queue, maintaining heap order.
         * Duplicates are allowed.
         * @param x the item to insert.
         */
        public void insert( HeapElement x )
        {
            if( isFull( ) )
            {
                HeapElement [] tempArray = new HeapElement[array.length*2];
                System.arraycopy(array, 0, tempArray, 0, array.length);
                array = tempArray;
            }

			currentSize++;
            array[currentSize] = x;
			percolateUp(currentSize);
        }

	public void decreaseKeyInsert(HeapElement x, int newKey)
	{
		x.decreasePriority(newKey);
		if(x.getHeapPos() <= 0)
			insert(x);
		else
			percolateUp(x.getHeapPos());
	}
        /**
         * Find the smallest item in the priority queue.
         * @return the smallest item, or null, if empty.
         */
        public HeapElement findMin( )
        {
            if( isEmpty( ) )
                return null;
            return array[ 1 ];
        }

        /**
         * Remove the smallest item from the priority queue.
         * @return the smallest item, or null, if empty.
         */
        public HeapElement deleteMin( )
        {
            if( isEmpty( ) )
                return null;

            HeapElement minItem = findMin( );
            array[ 1 ] = array[ currentSize-- ];
            percolateDown( 1 );
			minItem.setHeapPos(-1);
			
            return minItem;
        }

        /**
         * Establish heap order property from an arbitrary
         * arrangement of items. Runs in linear time.
         */
        private void buildHeap( )
        {
            for( int i = currentSize / 2; i > 0; i-- )
                percolateDown( i );
        }

        /**
         * Test if the priority queue is logically empty.
         * @return true if empty, false otherwise.
         */
        public boolean isEmpty( )
        {
            return currentSize == 0;
        }

        /**
         * Test if the priority queue is logically full.
         * @return true if full, false otherwise.
         */
        public boolean isFull( )
        {
            return currentSize == array.length - 1;
        }

        /**
         * Make the priority queue logically empty.
         */
        public void makeEmpty( )
        {
            currentSize = 0;
        }

        private static final int DEFAULT_CAPACITY = 100;

        private int currentSize;      // Number of elements in heap
        private HeapElement [ ] array; // The heap array

		private void percolateUp( int hole)
		{
			HeapElement tmp = array[ hole ];
            for( ; hole > 1 && tmp.comparePriority( array[ hole / 2 ] ) < 0; hole /= 2 )
            {
                array[ hole ] = array[ hole / 2 ];
                array[hole].setHeapPos(hole);
            }
            array[ hole ] = tmp;
            array[ hole ].setHeapPos(hole);
		}			
		
        /**
         * Internal method to percolate down in the heap.
         * @param hole the index at which the percolate begins.
         */
        private void percolateDown( int hole )
        {
			int child;
			HeapElement tmp = array[ hole ];

			for( ; hole * 2 <= currentSize; hole = child )
	        {
				child = hole * 2;
				if( child != currentSize && array[ child + 1 ].comparePriority( array[ child ] ) < 0 )
					child++;
				if( array[ child ].comparePriority( tmp ) < 0 )
                {
					array[ hole ] = array[ child ];
					array[hole].setHeapPos(hole);
                }
                else
					break;
            }
			array[ hole ] = tmp;
            array[hole].setHeapPos(hole);
        }
    }
